/// CSparse: a Concise Sparse matrix package.
/// Copyright (c) 2006, Timothy A. Davis.
/// http://www.cise.ufl.edu/research/sparse/CSparse
///
/// CSparse is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.
///
/// CSparse is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
/// Lesser General Public License for more details.
///
/// You should have received a copy of the GNU Lesser General Public
/// License along with this Module; if not, write to the Free Software
/// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
part of edu.emory.mathcs.csparse;

/// Applies a Householder reflection to a dense vector,
/// x = (I - beta*v*v')*x.
///
/// [V] column-compressed matrix of Householder vectors.
/// [i] v = V(:,i), the ith column of V.
/// [x] vector of size m. Returns true if successful, false on error.
bool happly(Matrix V, int i, double beta, Float64List x) {
  Int32List Vp, Vi;
  Float64List Vx;
  double tau = 0.0;
  if (!csc(V) || x == null) {
    return (false); // check inputs
  }
  Vp = V.p;
  Vi = V.i;
  Vx = V.x;
  for (int p = Vp[i]; p < Vp[i + 1]; p++) // tau = v'*x
  {
    tau += Vx[p] * x[Vi[p]];
  }
  tau *= beta; // tau = beta*(v'*x)
  for (int p = Vp[i]; p < Vp[i + 1]; p++) // x = x - v*tau
  {
    x[Vi[p]] -= Vx[p] * tau;
  }
  return true;
}
